/**
 * @file brightness.c
 *
 * Simple Brightness/Backlight control for the EMAC PPC-E7
 * and SoM-9307. Implements a slider application in GTK+
 * that directly controls the backlight through its sysfs
 * interface. Tested under Xfbdev/Matchbox.
 *
 * @author EMAC, Inc. <support@emacinc.com>
 *
 * Copyright (C) 2009 EMAC, Inc.
 *
 */
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <dirent.h>

#include <linux/ioctl.h>
#include <sys/ioctl.h>

#define SYSCLASS 	"/sys/class/backlight/"
#define SYS_STATE_ON  0
#define SYS_STATE_OFF 4
static char *SYS_BRIGHTNESS = NULL;
static char *SYS_MAXBRIGHTNESS = NULL;
static char *SYS_POWER = NULL;

static char *get_sysclass_bl(void)
{
	DIR *dp;
	char *dentry;
	struct dirent *d;

	if((dp = opendir(SYSCLASS)) == NULL) {
		fprintf(stderr, "unable to open %s", SYSCLASS);
		return NULL;
	}

	while((d = readdir(dp))) {
		if (!(strcmp(d->d_name, ".") == 0) &&
			!(strcmp(d->d_name, "..") == 0))
			asprintf(&dentry, "%s%s", SYSCLASS, d->d_name);
	}
	closedir(dp);
	return dentry;
}

static void setup_sysclass(void)
{
	char *bl_dev;
	
	bl_dev = get_sysclass_bl();
	
	SYS_BRIGHTNESS = g_strdup_printf("%s/brightness", bl_dev);
	SYS_MAXBRIGHTNESS = g_strdup_printf("%s/max_brightness", bl_dev);
	SYS_POWER = g_strdup_printf("%s/power", bl_dev);
}

int sysclass_set_level(int level)
{
	FILE *f_light;
	FILE *f_max;
	int val, maxlevel;

	if ( level > 255 ) level = 255;
	if ( level < 1 ) level = 1;

	f_max = fopen(SYS_MAXBRIGHTNESS, "r");
	if (f_max != NULL)
	{
		fscanf(f_max,"%d", &maxlevel);
		fclose(f_max);
	}
	val = ( level == 1 ) ? 1 : ( level * maxlevel ) / 255;
	if (val > maxlevel)
	{
		val = maxlevel;
	}
	f_light = fopen(SYS_BRIGHTNESS, "w");
	if (f_light != NULL)
	{
		fprintf(f_light,"%d\n", val);
		fclose(f_light);
	}
	else
		return -1;

	f_light = fopen(SYS_POWER, "w");
	if (f_light != NULL)
	{
		fprintf(f_light,"%d\n",  level ? SYS_STATE_ON : SYS_STATE_OFF);
		fclose(f_light);
	}

	return level;
}

int sysclass_get_level(void)
{
	FILE *f_light;
	FILE *f_max;
	float level, maxlevel, factor;

	f_max = fopen(SYS_MAXBRIGHTNESS, "r");
	if (f_max != NULL)
	{
		fscanf(f_max,"%f", &maxlevel);
		fclose(f_max);
	}
	factor = 255/maxlevel;
	f_light = fopen(SYS_BRIGHTNESS, "r");
	
	if (f_light != NULL)
	{
		fscanf(f_light,"%f", &level);
		fclose(f_light);
		level = level * factor;
		if (level > 255)
		{
			level = 255;
		}
		return (int)level;
	}

	return -1;
}

void cadjuster(GtkAdjustment *adj)
{
	sysclass_set_level(adj->value);
}

int main(int argc,char *argv[])
{
	GtkWidget *contrast_scroll;
	GtkWidget *window;
	GtkWidget *box;
	GtkWidget *carrow;
	GtkObject *cadjust;
	GtkWidget *frame_c;
	
	unsigned int i,coni;

	/*start of gtk section*/

	setup_sysclass();

	gtk_init(&argc,&argv);

	window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
	gtk_signal_connect(GTK_OBJECT(window), "destroy", GTK_SIGNAL_FUNC(gtk_main_quit),NULL);
	gtk_window_set_title (GTK_WINDOW (window), "Brightness");
	gtk_widget_set_size_request(window, 400, 60);
	
	box = gtk_hbox_new(FALSE, 10);
	coni = sysclass_get_level();

	gtk_container_add(GTK_CONTAINER(window), box);
	cadjust = gtk_adjustment_new(coni, 0x0, 0xFF, 1, 1, 0);

	contrast_scroll = gtk_hscale_new(GTK_ADJUSTMENT(cadjust));
	gtk_scale_set_draw_value(GTK_SCALE(contrast_scroll), FALSE);
	gtk_scale_set_digits(GTK_SCALE(contrast_scroll), 0);
	gtk_scale_set_value_pos(GTK_SCALE(contrast_scroll), GTK_POS_BOTTOM);
	gtk_signal_connect(GTK_OBJECT(cadjust), "value_changed", GTK_SIGNAL_FUNC(cadjuster), NULL);
	frame_c = gtk_frame_new ("Brightness");
	gtk_container_add(GTK_CONTAINER(frame_c), contrast_scroll);
	gtk_box_pack_start(GTK_BOX(box), frame_c, TRUE, TRUE, 0);
	gtk_widget_show(contrast_scroll);
	gtk_widget_show(frame_c);

	carrow = gtk_spin_button_new(GTK_ADJUSTMENT(cadjust), 1, 0);
	gtk_box_pack_start(GTK_BOX(box), carrow, TRUE, TRUE, 0);
	gtk_widget_show(carrow);

	gtk_widget_show(box);

	gtk_widget_show(window);

	gtk_main();

	exit(EXIT_SUCCESS);
}

