/**
 * @file serial.c
 *
 * This file provides an accessory functions for setting up and calling poll
 * on a serial port.
 *
 * @author EMAC, Inc. <support@emacinc.com>
 *
 * Copyright (C) 2009 EMAC, Inc.
 */
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#include <stdio.h>
#include <termios.h>
#include <string.h>
#include <fcntl.h>
#include <sys/types.h>
#include <poll.h>
#include <unistd.h>


/**
 * function to setup a given serial device to the specified parameters
 * @param fd the file descriptor of the open serial device
 * @param baud the desired baud rate (i.e. B115200)
 * @param handshake pass 1 to use hardware handshaking or 0 for no flow
 *   control
 * @return 0
 * @pre save the current termios flags of the device prior to calling this
 *   function
 */
int setup_serial(int fd, int baud, int handshake)
{
	struct termios newtio;
	tcgetattr(fd, &newtio);
	tcflag_t       cflags;

	cflags = CREAD;   // receiver enable
	if (handshake)
		cflags |= CRTSCTS;
	else
		cflags |= CLOCAL;  // local mode
	cflags |= CS8;     // 8 bits
	cflags |= baud;    // baud parameter

	newtio.c_cflag = cflags;
	newtio.c_iflag = IGNPAR | ICRNL;
	newtio.c_oflag = 0;
	newtio.c_lflag = 0;
	newtio.c_cc[VMIN] = 1;
	newtio.c_cc[VTIME] = 0;

	tcflush(fd, TCIOFLUSH); // Flush the device buffer
	tcsetattr(fd, TCSANOW, &newtio);

	return 0;
}


/**
 * function to call poll() on a serial port with the specified timeout waiting
 * for input on the port.
 * @param fd the file descriptor of the open serial device
 * @param timeout the desired timeout in milliseconds, -1 for infinite timeout
 * @return 0 on timeout, -1 on error, 1 on successful return -- input
 *   available
 */
int poll_serial(int fd, int timeout)
{
	struct pollfd fds[1];
	fds[0].fd = fd;
	fds[0].events = POLLIN;

	switch (poll(fds, 1, timeout))
	{
	case 0: /* timeout */
		return 0;
		break;
	case -1: /* poll error */
		return -1;
		break;
	default:
		if (fds[0].revents & POLLIN)
		{
			return 1;
		}
		/* non input event */
		return -1;
		break;
	}
	return -1;
}

