/**
 * @file serial_test.c
 *
 * This file provides an example program for accessing a serial port.
 * This example demonstrates I/O using the poll() function call.
 *
 * @author EMAC, Inc. <support@emacinc.com>
 *
 * Copyright (C) 2009 EMAC, Inc.
 */
/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include <termios.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <string.h>
#include <sys/signal.h>
#include <sys/types.h>
#include "serial.h"

#define BAUDRATE B115200
#define SERIALDEVICE "/dev/ttyS1"

static void clean_exit(int fd, struct termios *tio, int success);


int main(int argc, char *argv[])
{
	int numread;
	int fd;
	int keep_going;
	char buff[255];
	char *serial_device = SERIALDEVICE;

	struct termios oldtio;

	if (argc > 1) 
		serial_device = argv[1];

	printf("Testing on %s using poll()\n", serial_device);

	fd = open(serial_device, O_RDWR | O_NOCTTY | O_NONBLOCK);
	if (fd < 0) {
		perror(serial_device);
		return -1;
	}

	/* save old attributes and setup the serial port */
	tcgetattr(fd, &oldtio);
	setup_serial(fd, BAUDRATE, 0);

	sprintf(buff, "Type something and press Enter...\r\n");
	write(fd, buff, strlen(buff));

	keep_going = 1;

	while (keep_going) {
		/* poll the serial port with infinite timeout */
		if (poll_serial(fd, -1) <= 0) {
			fprintf(stderr, "Error: invalid return from poll()\n");
			clean_exit(fd, &oldtio, 0);
		}
		/* read all available data */
		if ((numread = read(fd, buff, sizeof(buff) - 1)) == -1) {
			perror("Error on read");
			clean_exit(fd, &oldtio, 0);
		}
		buff[numread] = '\0';
		/* parse and analyze serial data here if desired... */
		/* echo data read back to serial device and local stdout */
		printf("Received: %s: %d\n", buff, numread);
		write(fd, buff, numread);
		/* if newline is found in the buffer, quit gracefully */
		if (strchr(buff, '\n')) 
			keep_going = 0;
	}

	sprintf(buff, "\rDone\r\n");
	write(fd, buff, strlen(buff));
	tcdrain(fd);

	printf("Exiting...\n");
	clean_exit(fd, &oldtio, 1);
	/* should not reach here */
	exit(EXIT_FAILURE);
}


/**
 * function to cleanly exit the application. Restores original termios
 * settings to file descriptor and calls exit() with success or failure code.
 * @param fd the file descriptor of the open terminal device
 * @param tio the original termios settings to restore
 * @param success pass 1 for successful exit code 0 for failure
 * @post the program is terminated with exit()
 */
static void clean_exit(int fd, struct termios *tio, int success)
{
	tcflush(fd, TCIOFLUSH);
	tcsetattr(fd, TCSANOW, tio);
	close(fd);
	if (success)
		exit(EXIT_SUCCESS);
	exit(EXIT_FAILURE);
}

